// Modelado del edificio conocido como "El Polvorn"
// en Sigenza, Guadalajara. 
// Autor: Marcos Nieto (mnieto@arrakis.es)
// Version: 1.1 (diciembre 2001)
// Reconstruccin molino original. Se intentan suprimir cambios
//     - posiblemente se le aadiese la cornisa superior, que parece
//       postiza. Altura aadida a conjunto: unos 0.3 m
//     - el tejado de tejas sustituy el original, una cubierta de madera
//       de mayor altura e inclinacin.
// Posibilidades:
//     - la puerta y ventana del molino estn en un principio orientadas al
//       "sur geogrfico" de la imagen (como es norma entre molinos)
//     - es posible definir el nmero de hitos de amarre que se quieren
//       y la distancia a la que estn situados gracias a las variables
//       hito_numero e hito_separacion (distancia desde pared molino
//       a hito de amarre)
//     - el conjunto rota sobre su eje (excepto paredes) de modo que es
//       posible fingir que el viento proviene de una direccin concreta
//       gracias a la variable "encarar_viento" 
//     - se puede mover el conjunto completo (molino + hitos de amarre) con las
//       variables mover_conjunto_x, mover_conjunto_y y mover_conjunto_z
//     - se puede rotar el conjunto completo respecto a SU eje con las
//       variables rotar_conjunto_x, rotar_conjunto_y y rotar_conjunto_z
//     - se puede escalar el conjunto completo usando el "Factor_escala_conjunto"
#include "colors.inc"
#include "stones.inc"
#include "woods.inc"

// Ejes de coordenadas de Pov-Ray
//                           / \
//         ^                /   \
//       Y |               /-----\
//         |               |     |
//         | / Z           |  -  |
//         |/              | | | |
//         +-----> X       | | | |           

// ZONA DECLARACIONES QUE PUEDEN SER EXTERNAS
// SUMINISTRAMOS VALORES POR OMISION

                                      
                                       
// angulo que debe rotar la parte mvil del molino para encarar el viento. Posibles valores: 
//   0 --> viento procedente del este
//  90 --> viento procedente del "sur geogrfico" (puerta y ventana)
// 180 -->    "       "       "  oeste
// 270 -->    "       "       "  norte
#ifndef (encarar_viento)
  #declare encarar_viento = 30;     // ligeramente sur-este
#end  
                                        
#ifndef (zocalo_altura)
   #declare zocalo_altura = 0.39; 
#end
#ifndef (zocalo_radio)
   #declare zocalo_radio = 2.85;
#end
#ifndef (cuerpo_altura)
   #declare cuerpo_altura = 4.11;
#end
#ifndef (cuerpo_radio)
   #declare cuerpo_radio = 2.7;
#end

#declare Exterior = union {   
// hago una unin con todos los cilindros "exteriores" 
        // zocalo: posee 0,39 m de altura y 5,7 m de diametro
        cylinder {
                <0,0,0>,                              // centro cara inferior
                <0,zocalo_altura,0>,                  // centro cara superior         
                zocalo_radio                          // radio
                pigment { color Yellow }       
        }

        // cuerpo edificio: es de sillera; posee 4,11  m de altura y 5,4 m de diametro
        cylinder {
                <0,zocalo_altura,0>,                  // centro cara inferior
                <0,zocalo_altura + cuerpo_altura,0>,  // centro cara superior
                cuerpo_radio                          // radio
                open                                  // abierto en los extremos
              pigment { color Yellow }                
        }       

} // fin union     


#declare espesor_muros = 0.8;
#declare vaciado_altura =  zocalo_altura + cuerpo_altura;
#declare vaciado_radio = cuerpo_radio - espesor_muros;
#declare hueco_puerta_altura = 1.7;   
#declare hueco_puerta_anchura = 0.85;
#declare ventana_alfeizar = 3.2;
#declare parche = 0.15;     // apao que tengo que recrecer el radio del hueco interior
                            // para asegurar la interpenetracin con huecos de puerta y ventana

#declare Vaciado = union {
        // hueco interior: asumo que es cilindrico y de radio constante de 2.05 m, lo que nos
        //                 da un diametro de 4.1, lo que supone muros de 0.8 m de espesor
        cylinder {
                <0,0,0>,                // centro cara inferior
                <0,vaciado_altura,0>,   // centro cara superior         
                vaciado_radio + parche  // radio
                open                    // abierto en los extremos
                
        }
      
        // hueco_puerta: situada en la cara sur, tiene 1.7 m de altura y 0.85 m de anchura
        // la situo a 0.3 m del suelo. Para que atraviese la pared (0.8m de espesor) la
        // asigno arbitrariamente 0.85m de espesor, para que haya zona de interseccin con 
        // volumen interior
        box {
                <-(hueco_puerta_anchura/2),0,-(espesor_muros + 0.05)>,  // esquina cercana inferior izquierda
                <hueco_puerta_anchura/2,hueco_puerta_altura,0>              // esquina lejana superior derecha
                translate <0,zocalo_altura,-1.9>
        }          
                   
            
        // ventana: situada en la cara sur, tiene 0.55 m de altura y 0.48 m de anchura
        // situada a 3.2 m del suelo. Para que atraviese la pared (0.8m de espesor) la
        // asigno arbitrariamente 0.85m de espesor, para que haya zona de interseccin con 
        // volumen interior
        box {
                <-0.24,0,-(espesor_muros + 0.05)>,  // esquina cercana inferior izquierda
                <0.24,0.55,0>                       // esquina lejana superior derecha
                translate <0,ventana_alfeizar,-1.9>
        }                
} // fin union

// puerta

#declare cerradura_anchura = 0.2;
#declare cerradura_altura = 0.2; 
#declare cerradura_grosor = 0.05;
#declare hueco_cerradura_altura = 0.05;
#declare hueco_cerradura_anchura = 0.02;
#declare Altura_puerta = hueco_puerta_altura - 0.1;
#declare Anchura_puerta = hueco_puerta_anchura - 0.05;
#declare Espesor_puerta = 0.1;
// angulo de apertura puerta molino
//   0 --> cerrada
//  45 --> entreabierta hacia afuera
// -45 --> entreabierta hacia adentro
#ifndef (Apertura_puerta)
  #declare Apertura_puerta = 0;
#end  

#declare Puerta = union {
   box {
     <0,0,-(Espesor_puerta/2)>,  // esquina cercana inferior izquierda
     <Anchura_puerta,Altura_puerta,Espesor_puerta/2>   // esquina lejana superior derecha
     texture { T_Wood1 } 
   }
   difference {
     box {        // cerradura
// las coordenadas de la cerradura seran relativas a la puerta  
        <Anchura_puerta-cerradura_anchura,Altura_puerta/2,-(Espesor_puerta/2+cerradura_grosor)>
        <Anchura_puerta,(Altura_puerta/2)+cerradura_altura,-Espesor_puerta/2>   
        pigment { color White }
     }
     box {        // hueco cerradura
        <Anchura_puerta-cerradura_anchura/2,Altura_puerta/2+cerradura_altura/2,-(Espesor_puerta/2+cerradura_grosor+0.01)>
        <Anchura_puerta-cerradura_anchura/2+hueco_cerradura_anchura,Altura_puerta/2+cerradura_altura/2+hueco_cerradura_altura,-Espesor_puerta/2>           
     }
   }   

}                               
                       

// "paredes" (con puerta completa) del edificio 
#declare Paredes = union {
    difference {       
        object  { Exterior  }
        object  { Vaciado   }
    }
    object { Puerta    
      rotate <0,Apertura_puerta,0>
      translate <-(Anchura_puerta/2),zocalo_altura,-cuerpo_radio> 
    }   
} // fin union                                                            



// hito de amarre: servian para sujetar el palo de gobierno. En La Mancha
// son 9 y en "El Polvorn" son 11. Se encuentran a una distancia variable
// del muro del molino (4-5 m). Si tomamos la distancia como 5 m, se encontraran
// a 7.7 m del centro del molino, dispuestos a 32.72 entre s, separados por
// un arco de 4.39 m
#ifndef (hito_numero)
  #declare hito_numero = 11;          // numero de hitos de amarre
#end
#ifndef (hito_separacion)
  #declare hito_separacion = 5;       // distancia desde el muro del molino al hito de amarre
#end  
#declare hito_distancia = hito_separacion + cuerpo_radio;   // distancia eje-molino al hito
#declare Hito_de_amarre = union {
// parte semienterrada: 20 cm bajo tierra y
// 10 cm sobre ella. Le aado 0.01 para 
// asegurar que existe un sobrecruzamiento
// entre ambos volmenes
   cylinder {
      <0,-0.2,0>        // centro cara inferior
      <0,0.1+0.01,0>    // centro cara superior
      0.25              // radio
   }
// parte donde se produce el amarre propiamente dicho.
// Tiene forma de tronco de cono invertido, de 20 cm
// de altura.
   cone {
      <0,0.1,0>, 0.15   // centro y radio extremo inferior
      <0,0.3,0>, 0.2    // centro y radio extremo superior
   }
}


                
// A continuacin modelamos una serie de elementos que eventualmente debern
// moverse solidariamente. Por el momento estos sern:
//   - el armazn cnico que constituye la cubierta
//   - el gobierno, madero cuyo giro y sujeccin a los hitos de amarre
//     permite orientar al molino en la direccin deseada
//   - el eje del molino, elemento que transmite el movimiento de las
//     aspas a la compleja maquinaria interior


// armazn cnico: actualmente tiene 1,3 m de altura y unos 5,8 m de diametro
// Originalmente debio de ser de madera y ahora es de tejas.
// Para permitir que est "hueco" le resto otro cono con 0.1 m menos
// de radio y 0.01 m menos de altura.
// Le declaro un "centro de cara inferior" de coord. Y por debajo de 
// la altura alcanzada por el edificio para permitir algo del vuelo a la cubierta
                                             
                                             

#declare cubierta_empieza = (zocalo_altura + cuerpo_altura) - 0.2;
#declare cubierta_acaba=(((zocalo_altura+cuerpo_altura)*(hito_separacion+cuerpo_radio))/hito_separacion);
#declare altura_cubierta = cubierta_acaba - cubierta_empieza;
#declare cubierta_radio = cuerpo_radio + 0.25;

#declare cubierta = difference {
   cone {
      <0,cubierta_empieza,0>, cubierta_radio   // centro cara inferior
      <0,cubierta_acaba,0>, 0                  // centro cara superior: vertice
   }
   cone {
      <0,cubierta_empieza,0>, cubierta_radio - 0.1   // centro cara inferior
      <0,cubierta_acaba - 0.1,0>, 0                  // centro cara superior: vertice
   }  
      pigment { color Sienna }
} // fin difference


// gobierno: eje gracias al cual se puede rotar sobre el anillo el armazn
// cnico. Le asigno la longitud existente entre la cspide del armazn 
// cnico y un hito de amarre. En la imagen aparece por encima de la cubierta
// pero realmente estaba por abajo de sta.
#declare gobierno_ancho = 0.2;
#declare gobierno_alto = 0.2;
#declare gobierno_largo = sqrt(hito_distancia*hito_distancia + cubierta_acaba*cubierta_acaba) - 1;

#declare gobierno_angulo_armazon = degrees(asin(altura_cubierta/sqrt(altura_cubierta*altura_cubierta + cubierta_radio*cubierta_radio)));

# declare gobierno = box {
    <-gobierno_largo, 0, -gobierno_ancho>,     // esquina cercana inferior izquierda
    <0,gobierno_alto,0>                        // esquina lejana superior derecha   
    rotate <0,0,gobierno_angulo_armazon>
    translate <0,cubierta_acaba - gobierno_ancho - 0.01, gobierno_ancho/2>

    pigment { color Copper }
}

                      
// eje del molino: realmente es el eje en torno al cual giran las aspas      
#declare eje_radio = 0.2;
#declare eje_largo = 5.5;
#declare eje_angulo = 75;
#declare Eje =  cylinder {
                <0,0,0>,             // centro cara inferior
                <0,eje_largo,0>,     // centro cara superior         
                eje_radio            // radio
}                 

// union eje-aspas 
#declare union_eje_aspas_grueso = 0.5;
#declare union_eje_aspas_centro = 0.3;
#declare union_eje_aspas = difference { 
  cylinder { 
    <0,eje_largo,0>,    // centro cara inferior
    <0,eje_largo + union_eje_aspas_grueso,0>,    // centro cara superior
    union_eje_aspas_grueso           // radio
  }
  torus { 
    1.5,       // radio exterior
    0.7        // radio interior    
    translate <0,0.5,0>
  }  
}

// pijote: extremo que sobresale del eje del molino, sobre la base de
// este apndice se colocaba una garrucha en funcin directa con la
// elevacin de las aspas
#declare pijote = cone {
    <0,eje_largo + union_eje_aspas_grueso,0>, 0.3     // centro y radio extremo inferior
    <0,eje_largo + union_eje_aspas_grueso + 1,0>, 0.1   // centro y radio extremo superior
}

// aspas
#declare aspa_grosor = 0.1;
#declare aspa_largo = 5.1;
#declare Aspa = cylinder {
    <0,0,0>,             // centro cara inferior
    <0,aspa_largo,0>,    // centro cara superior
    aspa_grosor          // radio   
    rotate <0,0,90>      // tumbo el aspa
    pigment { color Copper }    
}

// soporte lonas: denominacion mia para el conjunto de elementos que componen el
// bastidor donde se sujetan las velas o lonas. De las fotos tomo 4 elementos
// longitudinales y numerosos transversales
#declare soporte_trans_grosor = 0.05;
#declare soporte_trans_largo = 2.0;
#declare soporte_trans_numero = 10;
#declare soporte_long_grosor = 0.05;
#declare soporte_long_largo = 3.0;
#declare soporte_long_numero = 4;
#declare soporte_inicio = 2.0;             // longitud de aspa a partir de la cual
                                           // se insertan los soportes transversales
#declare soporte_trans_separacion = 0.5;   // separacin entre soportes transversales

#declare soporte_trans = cylinder {
    <0,0,0>,                      // centro cara inferior
    <0,soporte_trans_largo,0>,    // centro cara superior
    soporte_trans_grosor          // radio   
    rotate <0,0,90>               // tumbo el soporte transversal
}                

#declare soporte_long = cylinder {
    <0,0,0>,                      // centro cara inferior
    <0,soporte_long_largo,0>,     // centro cara superior
    soporte_long_grosor           // radio   
    rotate <90,0,0>               // tumbo el soporte transversal
}

// File         : TheTableCloth.mdl
// Size (in K)  : 11KB
// Date         : 16 April 2000 
// Author       : Stewart Cowley
// AuthorEmail  : stewartcowley@email.msn.com
#declare Cloth = 
   material  // Cloth
   {   
      texture
      {      
         pigment
         {
            color rgb <1.0, 1.0, 0.854167>
         }      
         normal
         {
            wrinkles , 0.15
            scale  0.1
         }      
         finish
         {
            ambient 0.1
            phong 0.064733
            specular 0.079133
         }
      }
   }

//lona
#declare lona = union{
//
// ********  OBJECTS  *******
//

  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-8.000000, -5.000000, 0.000000>, <-7.000000, -5.000000, 0.000000>, <-7.968941, -5.000000, -0.073377>, <-5.886635, -5.000000, -0.048918>, 
      <-8.000000, -4.000000, 0.000000>, <-7.000000, -4.000000, 0.000000>, <-7.968941, -4.000000, -0.073377>, <-5.886635, -4.000000, -0.048918>, 
      <-8.000000, -3.000000, 0.000000>, <-7.000000, -3.000000, 0.000000>, <-7.968941, -3.000000, -0.073377>, <-5.886635, -3.000000, -0.048918>, 
      <-8.000000, -2.000000, 0.000000>, <-7.000000, -2.000000, 0.000000>, <-7.968941, -2.000000, -0.073377>, <-5.886635, -2.000000, -0.048918>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-8.000000, -2.000000, 0.000000>, <-7.000000, -2.000000, 0.000000>, <-7.968941, -2.000000, -0.073377>, <-5.886635, -2.000000, -0.048918>, 
      <-8.000000, -1.000000, 0.000000>, <-7.000000, -1.000000, 0.000000>, <-7.968941, -1.000000, -0.073377>, <-5.886635, -1.000000, -0.048918>, 
      <-8.000000, 0.000000, 0.000000>, <-7.000000, 0.000000, 0.000000>, <-7.968941, 0.000000, -0.073377>, <-5.886635, 0.000000, -0.048918>, 
      <-8.000000, 1.000000, 0.000000>, <-7.000000, 1.000000, 0.000000>, <-7.968941, 1.000000, -0.073377>, <-5.886635, 1.000000, -0.048918>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-8.000000, 1.000000, 0.000000>, <-7.000000, 1.000000, 0.000000>, <-7.968941, 1.000000, -0.073377>, <-5.886635, 1.000000, -0.048918>, 
      <-8.000000, 2.000000, 0.000000>, <-7.000000, 2.000000, 0.000000>, <-7.968941, 2.000000, -0.073377>, <-5.886635, 2.000000, -0.048918>, 
      <-8.000000, 3.000000, 0.000000>, <-7.000000, 3.000000, 0.000000>, <-7.968941, 3.000000, -0.073377>, <-5.886635, 3.000000, -0.048918>, 
      <-8.000000, 4.000000, 0.000000>, <-7.000000, 4.000000, 0.000000>, <-7.968941, 4.000000, -0.073377>, <-5.886635, 4.000000, -0.048918>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-5.886635, -5.000000, -0.048918>, <-3.804329, -5.000000, -0.024459>, <-2.767640, -5.000000, 0.073377>, <-1.785985, -5.000000, 0.024459>, 
      <-5.886635, -4.000000, -0.048918>, <-3.804329, -4.000000, -0.024459>, <-2.767640, -4.000000, 0.073377>, <-1.785985, -4.000000, 0.024459>, 
      <-5.886635, -3.000000, -0.048918>, <-3.804329, -3.000000, -0.024459>, <-2.767640, -3.000000, 0.073377>, <-1.785985, -3.000000, 0.024459>, 
      <-5.886635, -2.000000, -0.048918>, <-3.804329, -2.000000, -0.024459>, <-2.767640, -2.000000, 0.073377>, <-1.785985, -2.000000, 0.024459>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-5.886635, -2.000000, -0.048918>, <-3.804329, -2.000000, -0.024459>, <-2.767640, -2.000000, 0.073377>, <-1.785985, -2.000000, 0.024459>, 
      <-5.886635, -1.000000, -0.048918>, <-3.804329, -1.000000, -0.024459>, <-2.767640, -1.000000, 0.073377>, <-1.785985, -1.000000, 0.024459>, 
      <-5.886635, 0.000000, -0.048918>, <-3.804329, 0.000000, -0.024459>, <-2.767640, 0.000000, 0.073377>, <-1.785985, 0.000000, 0.024459>, 
      <-5.886635, 1.000000, -0.048918>, <-3.804329, 1.000000, -0.024459>, <-2.767640, 1.000000, 0.073377>, <-1.785985, 1.000000, 0.024459>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-5.886635, 1.000000, -0.048918>, <-3.804329, 1.000000, -0.024459>, <-2.767640, 1.000000, 0.073377>, <-1.785985, 1.000000, 0.024459>, 
      <-5.886635, 2.000000, -0.048918>, <-3.804329, 2.000000, -0.024459>, <-2.767640, 2.000000, 0.073377>, <-1.785985, 2.000000, 0.024459>, 
      <-5.886635, 3.000000, -0.048918>, <-3.804329, 3.000000, -0.024459>, <-2.767640, 3.000000, 0.073377>, <-1.785985, 3.000000, 0.024459>, 
      <-5.886635, 4.000000, -0.048918>, <-3.804329, 4.000000, -0.024459>, <-2.767640, 4.000000, 0.073377>, <-1.785985, 4.000000, 0.024459>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-1.785985, -5.000000, 0.024459>, <-0.804329, -5.000000, -0.024459>, <0.256818, -5.000000, 0.036688>, <1.177709, -5.000000, 0.047773>, 
      <-1.785985, -4.000000, 0.024459>, <-0.804329, -4.000000, -0.024459>, <0.256818, -4.000000, 0.036688>, <1.177709, -4.000000, 0.047773>, 
      <-1.785985, -3.000000, 0.024459>, <-0.804329, -3.000000, -0.024459>, <0.256818, -3.000000, 0.036688>, <1.177709, -3.000000, 0.047773>, 
      <-1.785985, -2.000000, 0.024459>, <-0.804329, -2.000000, -0.024459>, <0.256818, -2.000000, 0.036688>, <1.171499, -2.000000, 0.053983>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-1.785985, -2.000000, 0.024459>, <-0.804329, -2.000000, -0.024459>, <0.256818, -2.000000, 0.036688>, <1.171499, -2.000000, 0.053983>, 
      <-1.785985, -1.000000, 0.024459>, <-0.804329, -1.000000, -0.024459>, <0.256818, -1.000000, 0.036688>, <1.177709, -1.000000, 0.047773>, 
      <-1.785985, 0.000000, 0.024459>, <-0.804329, 0.000000, -0.024459>, <0.256818, 0.000000, 0.036688>, <1.177709, 0.000000, 0.047773>, 
      <-1.785985, 1.000000, 0.024459>, <-0.804329, 1.000000, -0.024459>, <0.256818, 1.000000, 0.036688>, <1.165384, 1.000000, 0.078442>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <-1.785985, 1.000000, 0.024459>, <-0.804329, 1.000000, -0.024459>, <0.256818, 1.000000, 0.036688>, <1.165384, 1.000000, 0.078442>, 
      <-1.785985, 2.000000, 0.024459>, <-0.804329, 2.000000, -0.024459>, <0.256818, 2.000000, 0.036688>, <1.165479, 2.000000, 0.096691>, 
      <-1.785985, 3.000000, 0.024459>, <-0.804329, 3.000000, -0.024459>, <0.256818, 3.000000, 0.036688>, <1.165479, 3.000000, 0.096691>, 
      <-1.785985, 4.000000, 0.024459>, <-0.804329, 4.000000, -0.024459>, <0.256818, 4.000000, 0.036688>, <1.177709, 4.000000, 0.047773>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <1.177709, -5.000000, 0.047773>, <2.098599, -5.000000, 0.058857>, <3.049681, -5.000000, 0.058857>, <4.178090, -5.000000, 0.040513>, 
      <1.177709, -4.000000, 0.047773>, <2.098599, -4.000000, 0.058857>, <3.049681, -4.000000, 0.058857>, <4.178090, -4.000000, 0.040513>, 
      <1.177709, -3.000000, 0.047773>, <2.098599, -3.000000, 0.058857>, <3.049681, -3.000000, 0.058857>, <4.178090, -3.000000, 0.040513>, 
      <1.171499, -2.000000, 0.053983>, <2.098599, -2.000000, 0.058857>, <3.049681, -2.000000, 0.058857>, <4.165670, -2.000000, 0.052934>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <1.171499, -2.000000, 0.053983>, <2.098599, -2.000000, 0.058857>, <3.049681, -2.000000, 0.058857>, <4.165670, -2.000000, 0.052934>, 
      <1.177709, -1.000000, 0.047773>, <2.098599, -1.000000, 0.058857>, <3.049681, -1.000000, 0.058857>, <4.178090, -1.000000, 0.040513>, 
      <1.177709, 0.000000, 0.047773>, <2.098599, 0.000000, 0.058857>, <3.049681, 0.000000, 0.058857>, <4.178090, 0.000000, 0.040513>, 
      <1.165384, 1.000000, 0.078442>, <2.086369, 1.000000, 0.107775>, <3.049681, 1.000000, 0.058857>, <4.165670, 1.000000, 0.052934>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <1.165384, 1.000000, 0.078442>, <2.086369, 1.000000, 0.107775>, <3.049681, 1.000000, 0.058857>, <4.165670, 1.000000, 0.052934>, 
      <1.165479, 2.000000, 0.096691>, <2.074140, 2.000000, 0.156693>, <3.049681, 2.000000, 0.058857>, <4.178090, 2.000000, 0.040513>, 
      <1.165479, 3.000000, 0.096691>, <2.074140, 3.000000, 0.156693>, <3.061910, 3.000000, 0.009940>, <4.184205, 3.000000, 0.016054>, 
      <1.177709, 4.000000, 0.047773>, <2.098599, 4.000000, 0.058857>, <3.049681, 4.000000, 0.058857>, <4.178090, 4.000000, 0.040513>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <4.178090, -5.000000, 0.040513>, <5.306499, -5.000000, 0.022169>, <6.061910, -5.000000, -0.038978>, <7.049681, -5.000000, 0.009940>, 
      <4.178090, -4.000000, 0.040513>, <5.306499, -4.000000, 0.022169>, <6.237474, -4.000000, 0.101348>, <7.049681, -4.000000, 0.009940>, 
      <4.178090, -3.000000, 0.040513>, <5.306499, -3.000000, 0.022169>, <6.061910, -3.000000, -0.038978>, <7.049681, -3.000000, 0.009940>, 
      <4.165670, -2.000000, 0.052934>, <5.306499, -2.000000, 0.022169>, <6.061910, -2.000000, -0.038978>, <7.049681, -2.000000, 0.009940>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <4.165670, -2.000000, 0.052934>, <5.306499, -2.000000, 0.022169>, <6.061910, -2.000000, -0.038978>, <7.049681, -2.000000, 0.009940>, 
      <4.178090, -1.000000, 0.040513>, <5.306499, -1.000000, 0.022169>, <6.061910, -1.000000, -0.038978>, <7.049681, -1.000000, 0.009940>, 
      <4.178090, 0.000000, 0.040513>, <5.306499, 0.000000, 0.022169>, <6.061910, 0.000000, -0.038978>, <7.049681, 0.000000, 0.009940>, 
      <4.165670, 1.000000, 0.052934>, <5.306499, 1.000000, 0.022169>, <6.132799, 1.000000, -0.011406>, <7.049681, 1.000000, 0.009940>
    }
  }
  object {
    bicubic_patch { type 1 flatness 0.0100 u_steps 3 v_steps 3,
      <4.165670, 1.000000, 0.052934>, <5.306499, 1.000000, 0.022169>, <6.132799, 1.000000, -0.011406>, <7.049681, 1.000000, 0.009940>, 
      <4.178090, 2.000000, 0.040513>, <5.306499, 2.000000, 0.022169>, <6.203687, 2.000000, 0.016166>, <7.049681, 2.000000, 0.009940>, 
      <4.184205, 3.000000, 0.016054>, <5.306499, 3.000000, 0.022169>, <6.215917, 3.000000, -0.055961>, <7.049681, 3.000000, 0.009940>, 
      <4.178090, 4.000000, 0.040513>, <5.306499, 4.000000, 0.022169>, <6.061910, 4.000000, -0.038978>, <7.025222, 4.000000, -0.091623>
    }
  }
  material {  Cloth }
  scale <0.2, 0.25, 1.0>


} 






// declaro el conjunto "Aspa_completa" que consta del eje longitudinal del aspa mas el bastidor
// de elementos longitudinales y transversales donde se fija la vela o lona
#declare Aspa_completa = union {     
    object { Aspa }
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-soporte_inicio> } 
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*1)> }
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*2)> }
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*3)> }   
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*4)> }
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*5)> }
    object { soporte_trans translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + soporte_trans_separacion*6)> }
    object { soporte_long translate <-(soporte_trans_largo/2),union_eje_aspas_centro,-(soporte_inicio + 3.0)> } 
    object { soporte_long translate <soporte_trans_largo/2,union_eje_aspas_centro,-(soporte_inicio + 3.0)> } 
    object { lona    rotate < 0, 90, 90> translate <-0.1, 0.6, -3.5>    }
}

      


#declare Remate = union {
  
   object { Aspa_completa  translate <0,union_eje_aspas_centro,0> }     
   object { Aspa_completa  rotate <0,90,0>   translate <0,union_eje_aspas_centro,0> }
   object { Aspa_completa  rotate <0,180,0>  translate <0,union_eje_aspas_centro,0> }
   object { Aspa_completa  rotate <0,270,0>  translate <0,union_eje_aspas_centro,0> }      
   pigment { color Copper }        
 
}

          





                           


// ZONA DE INSTANCIAMIENTO

#declare hito_contador = 0;
#declare hito_incremento = 360/hito_numero;      // angulo de separacion entre hitos contiguos      
#declare hito_angulo = hito_contador;            
#ifndef (factor_giro_aspas)
    #declare factor_giro_aspas = 1.0;   // factor por el que multiplicamos la variable interna "clock"
#end                                    // para aumentar o disminuir la velocidad del giro

#declare Resto = union {
    // posiciono los 11 hitos de amarre en torno al molino
    #while (hito_contador < hito_numero)
        object { Hito_de_amarre translate <hito_distancia,0,0> rotate <0,hito_contador * hito_incremento,0> pigment {color White}}
        #declare hito_contador = hito_contador + 1;
    #end
    union {
        object { cubierta }
        object { gobierno }
    }
// esto sirve  para instanciar y rotar todo el conjunto eje+remate+pijote+aspas;       
    object {
       union {
          object { Eje  } 
          object { union_eje_aspas }
          object { pijote }   
// Realizo una primera rotacin para que comience en la postura de "cruz de San Andrs", para que no
// tape la puerta y la ventana de la fachada sur
          object { Remate rotate <0,45,0> rotate <0, factor_giro_aspas * clock *  360,0> translate <0,eje_largo,0> }
          
       } rotate <0,0,-eje_angulo>     
//       translate <-2.4,vaciado_altura,0>      asumo que el eje apoya en la mitad del muro
       translate <-(cuerpo_radio - espesor_muros/2),vaciado_altura,0>
       pigment { color Copper }      
    } rotate <0,encarar_viento,0>
} 

#declare Molino = union {
   object { Paredes }
   object { Resto }
}


